@@ -75,15 +75,15 @@ static struct {
 		offsetof(struct ctdb_tunable_list, rerecovery_timeout) },
 	{ "EnableBans", 1, false,
 		offsetof(struct ctdb_tunable_list, enable_bans) },
-	{ "DeterministicIPs", 0, false,
+	{ "DeterministicIPs", 0, true,
 		offsetof(struct ctdb_tunable_list, deterministic_public_ips) },
-	{ "LCP2PublicIPs", 1, false,
+	{ "LCP2PublicIPs", 1, true,
 		offsetof(struct ctdb_tunable_list, lcp2_public_ip_assignment) },
 	{ "ReclockPingPeriod", 60, true,
 		offsetof(struct ctdb_tunable_list,  reclock_ping_period) },
 	{ "NoIPFailback", 0, false,
 		offsetof(struct ctdb_tunable_list, no_ip_failback) },
-	{ "DisableIPFailover", 0, false,
+	{ "DisableIPFailover", 0, true,
 		offsetof(struct ctdb_tunable_list, disable_ip_failover) },
 	{ "VerboseMemoryNames", 0, false,
 		offsetof(struct ctdb_tunable_list, verbose_memory_names) },
@@ -105,7 +105,7 @@ static struct {
 		offsetof(struct ctdb_tunable_list, vacuum_max_run_time) },
 	{ "RepackLimit", 10*1000, false,
 		offsetof(struct ctdb_tunable_list, repack_limit) },
-	{ "VacuumLimit", 5*1000, false,
+	{ "VacuumLimit", 5*1000, true,
 		offsetof(struct ctdb_tunable_list, vacuum_limit) },
 	{ "VacuumFastPathCount", 60, false,
 		offsetof(struct ctdb_tunable_list, vacuum_fast_path_count) },
@@ -119,7 +119,7 @@ static struct {
 		offsetof(struct ctdb_tunable_list, deferred_attach_timeout) },
 	{ "AllowClientDBAttach", 1, false,
 		offsetof(struct ctdb_tunable_list, allow_client_db_attach) },
-	{ "RecoverPDBBySeqNum", 1, false,
+	{ "RecoverPDBBySeqNum", 1, true,
 		offsetof(struct ctdb_tunable_list, recover_pdb_by_seqnum) },
 	{ "DeferredRebalanceOnNodeAdd", 300, true,
 		offsetof(struct ctdb_tunable_list, deferred_rebalance_on_node_add) },
@@ -141,11 +141,11 @@ static struct {
 		offsetof(struct ctdb_tunable_list, db_size_warn) },
 	{ "PullDBPreallocation", 10*1024*1024, false,
 		offsetof(struct ctdb_tunable_list, pulldb_preallocation_size) },
-	{ "NoIPHostOnAllDisabled", 0, false,
+	{ "NoIPHostOnAllDisabled", 1, true,
 		offsetof(struct ctdb_tunable_list, no_ip_host_on_all_disabled) },
 	{ "Samba3AvoidDeadlocks", 0, true,
 		offsetof(struct ctdb_tunable_list, samba3_hack) },
-	{ "TDBMutexEnabled", 0, false,
+	{ "TDBMutexEnabled", 1, true,
 		offsetof(struct ctdb_tunable_list, mutex_enabled) },
 	{ "LockProcessesPerDB", 200, false,
 		offsetof(struct ctdb_tunable_list, lock_processes_per_db) },
@@ -153,7 +153,11 @@ static struct {
 		offsetof(struct ctdb_tunable_list, rec_buffer_size_limit) },
 	{ "QueueBufferSize", 1024, false,
 		offsetof(struct ctdb_tunable_list, queue_buffer_size) },
-	{ NULL, 0, true, }
+	{ "IPAllocAlgorithm", 2, false,
+		offsetof(struct ctdb_tunable_list, ip_alloc_algorithm) },
+	{ "AllowMixedVersions", 0, false,
+		offsetof(struct ctdb_tunable_list, allow_mixed_versions) },
+	{ .obsolete = true, }
 };
 
 void ctdb_tunable_set_defaults(struct ctdb_tunable_list *tun_list)
