@@ -22,15 +22,10 @@ from __future__ import print_function
 """Blackbox tests for ndrdump."""
 
 import os
+import re
 from samba.tests import BlackboxTestCase, BlackboxProcessError
 
-for p in ["../../../../../source4/librpc/tests",
-          "../../../../../librpc/tests"]:
-    data_path_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), p))
-    print(data_path_dir)
-    if os.path.exists(data_path_dir):
-        break
-
+data_path_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), "../../../../../source4/librpc/tests"))
 
 class NdrDumpTests(BlackboxTestCase):
     """Blackbox tests for ndrdump."""
@@ -95,6 +90,41 @@ class NdrDumpTests(BlackboxTestCase):
                          expected.encode('utf-8'))
         self.assertTrue(actual.endswith(b"dump OK\n"))
 
+    def test_ndrdump_upn_dns_info_ex(self):
+        with open(self.data_path(
+                'krb5pac_upn_dns_info_ex.txt')) as f:
+            expected = f.read()
+        data_path = self.data_path(
+            'krb5pac_upn_dns_info_ex.b64.txt')
+
+        try:
+            actual = self.check_output(
+                'ndrdump --debug-stdout -d0 krb5pac PAC_DATA struct '
+                '--validate --base64-input ' + data_path)
+        except BlackboxProcessError as e:
+            self.fail(e)
+
+        self.assertEqual(actual, expected.encode('utf-8'))
+
+    def test_ndrdump_upn_dns_info_ex_not_supported(self):
+        with open(self.data_path(
+                'krb5pac_upn_dns_info_ex_not_supported.txt')) as f:
+            expected = f.read()
+        data_path = self.data_path(
+            'krb5pac_upn_dns_info_ex_not_supported.b64.txt')
+
+        try:
+            # This PAC has been edited to remove the
+            # PAC_UPN_DNS_FLAG_HAS_SAM_NAME_AND_SID bit, so that we can
+            # simulate older versions of Samba parsing this structure.
+            actual = self.check_output(
+                'ndrdump --debug-stdout -d0 krb5pac PAC_DATA struct '
+                '--validate --base64-input ' + data_path)
+        except BlackboxProcessError as e:
+            self.fail(e)
+
+        self.assertEqual(actual, expected.encode('utf-8'))
+
     def test_ndrdump_with_binary_struct_number(self):
         expected = '''pull returned Success
     GUID                     : 33323130-3534-3736-3839-616263646566
@@ -141,7 +171,7 @@ dump OK
     def test_ndrdump_input_cmdline_short_struct_name_dump(self):
         expected = '''pull returned Buffer Size Error
 6 bytes consumed
-[0000] 61 62 63 64 65 66 67                               abcdefg ''' \
+[0000] 61 62 63 64 65 66 67                                abcdefg''' \
         '''
 '''
         try:
@@ -157,10 +187,10 @@ dump OK
     def test_ndrdump_input_cmdline_short_struct_name_print_fail(self):
         expected = '''pull returned Buffer Size Error
 6 bytes consumed
-[0000] 61 62 63 64 65 66 67                               abcdefg ''' \
+[0000] 61 62 63 64 65 66 67                                abcdefg''' \
         '''
 WARNING! 1 unread bytes
-[0000] 67                                                 g ''' \
+[0000] 67                                                  g''' \
     '''
 WARNING: pull of GUID was incomplete, therefore the parse below may SEGFAULT
     GUID                     : 64636261-6665-0000-0000-000000000000
@@ -182,7 +212,7 @@ WARNING! 53 unread bytes
 [0000] 00 FF 00 00 FF 00 00 00   00 09 00 00 00 08 00 33   ........ .......3
 [0010] 33 32 37 36 32 36 39 33   32 37 36 38 34 01 00 00   32762693 27684...
 [0020] 80 32 0D FF 00 00 FF 00   00 00 00 08 00 00 00 1C   .2...... ........
-[0030] F1 29 08 00 00                                     .)... ''' \
+[0030] F1 29 08 00 00                                      .)...''' \
         b'''
     clusapi_QueryAllValues: struct clusapi_QueryAllValues
         out: struct clusapi_QueryAllValues
@@ -301,6 +331,18 @@ dump OK
         # convert expected to bytes for python 3
         self.assertEqual(actual, expected.encode('utf-8'))
 
+    def test_ndrdump_fuzzed_drsuapi_DsaAddressListItem_V1(self):
+        expected = "Maximum Recursion Exceeded"
+        try:
+            self.check_output(
+                "ndrdump drsuapi 17 out --base64-input %s" %
+                self.data_path(
+                    "fuzzed_drsuapi_DsaAddressListItem_V1-in.b64.txt"))
+            self.fail("Input should have been rejected with %s" % expected)
+        except BlackboxProcessError as e:
+            if expected not in str(e):
+                self.fail(e)
+
     def test_ndrdump_fuzzed_drsuapi_DsReplicaAttribute(self):
         expected = open(self.data_path("fuzzed_drsuapi_DsReplicaAttribute.txt")).read()
         try:
@@ -313,12 +355,49 @@ dump OK
         # convert expected to bytes for python 3
         self.assertEqual(actual, expected.encode('utf-8'))
 
+    def test_ndrdump_Krb5ccache(self):
+        expected = open(self.data_path("../../../source3/selftest/"
+                                       "ktest-krb5_ccache-2.txt")).read()
+        try:
+            # Specify -d1 to match the generated output file, because ndrdump
+            # only outputs some additional info if this parameter is specified,
+            # and the --configfile parameter gives us an empty smb.conf to avoid
+            # extraneous output.
+            actual = self.check_output(
+                "ndrdump krb5ccache CCACHE struct "
+                "--configfile /dev/null -d1 --validate " +
+                self.data_path("../../../source3/selftest/"
+                               "ktest-krb5_ccache-2"))
+        except BlackboxProcessError as e:
+            self.fail(e)
+        # check_output will return bytes
+        # convert expected to bytes for python 3
+        self.assertEqual(actual, expected.encode('utf-8'))
+
+        expected = open(self.data_path("../../../source3/selftest/"
+                                       "ktest-krb5_ccache-3.txt")).read()
+        try:
+            # Specify -d1 to match the generated output file, because ndrdump
+            # only outputs some additional info if this parameter is specified,
+            # and the --configfile parameter gives us an empty smb.conf to avoid
+            # extraneous output.
+            actual = self.check_output(
+                "ndrdump krb5ccache CCACHE struct "
+                "--configfile /dev/null -d1 --validate " +
+                self.data_path("../../../source3/selftest/"
+                               "ktest-krb5_ccache-3"))
+        except BlackboxProcessError as e:
+            self.fail(e)
+        # check_output will return bytes
+        # convert expected to bytes for python 3
+        self.assertEqual(actual, expected.encode('utf-8'))
+
     # This is a good example of a union with an empty default
     # and no buffers to parse.
     def test_ndrdump_fuzzed_spoolss_EnumForms(self):
         expected_head = b'''pull returned Success
 WARNING! 2 unread bytes
-[0000] 00 00                                              .. ''' b'''
+[0000] 00 00                                               ..''' b'''
     spoolss_EnumForms: struct spoolss_EnumForms
         out: struct spoolss_EnumForms
             count                    : *
@@ -402,6 +481,10 @@ pull returned Success
 WARNING! orig bytes:29 validated pushed bytes:16
 WARNING! orig and validated differ at byte 0x04 (4)
 WARNING! orig byte[0x04] = 0xC6 validated byte[0x04] = 0x00
+-[0000] 40 F3 38 41 C6 92 87 04   00 00 00 00 00 00 00 06   @.8A.... ........
++[0000] 40 F3 38 41 00 00 00 00   00 00 00 00 00 00 00 00   @.8A.... ........
+-[0010] F5 FF 00 3C 3C 25 FF 70   16 1F A0 12 84            ...<<%.p .....
++[0010]                                                     EMPTY   BLOCK
 dump OK
 '''
         try:
@@ -417,13 +500,17 @@ dump OK
     def test_ndrdump_fuzzed_NULL_struct_ntlmssp_CHALLENGE_MESSAGE(self):
         expected =  open(self.data_path("fuzzed_ntlmssp-CHALLENGE_MESSAGE.txt")).read().encode('utf8')
         try:
-            actual = self.check_exit_code(
+            actual = self.check_output(
                 "ndrdump ntlmssp CHALLENGE_MESSAGE struct --validate --input " +\
-                "'AAAACwIAAAAAJwIAAAAAAAcAAAAAAAAAAIAbhG8uyk9dAL0mQE73MAAAAAAAAAAA' --base64-input",
-                1)
+                "'AAAACwIAAAAAJwIAAAAAAAcAAAAAAAAAAIAbhG8uyk9dAL0mQE73MAAAAAAAAAAA' --base64-input")
         except BlackboxProcessError as e:
             self.fail(e)
 
+        # Filter out the C source file and line number
+        regex = rb"\.\./\.\./librpc/ndr/ndr\.c:[0-9]+"
+        actual = re.sub(regex, b"", actual)
+        expected = re.sub(regex, b"", expected)
+
         self.assertEqual(actual, expected)
 
     # Test a print of NULL pointer in manually-written ndr_drsuapi.c
@@ -438,6 +525,19 @@ dump OK
 
         self.assertEqual(actual, expected)
 
+    def test_ndrdump_fuzzed_ndr_compression(self):
+        expected = 'pull returned Buffer Size Error'
+        command = (
+            "ndrdump drsuapi 3 out --base64-input "
+            "--input BwAAAAcAAAAGAAAAAwAgICAgICAJAAAAICAgIAkAAAAgIAAA//////8=")
+        try:
+            actual = self.check_exit_code(command, 2)
+        except BlackboxProcessError as e:
+            self.fail(e)
+        # check_output will return bytes
+        # convert expected to bytes for python 3
+        self.assertRegex(actual.decode('utf8'), expected + '$')
+
     def test_ndrdump_short_dnsProperty(self):
         expected = b'''pull returned Success
     dnsp_DnsProperty_short: struct dnsp_DnsProperty_short
