@@ -214,6 +214,16 @@ static NTSTATUS gse_context_init(TALLOC_CTX *mem_ctx,
 		goto err_out;
 	}
 
+#ifdef SAMBA4_USES_HEIMDAL
+	k5ret = gsskrb5_set_dns_canonicalize(false);
+	if (k5ret) {
+		DBG_ERR("gsskrb5_set_dns_canonicalize() failed (%s)\n",
+			error_message(k5ret));
+		status = NT_STATUS_INTERNAL_ERROR;
+		goto err_out;
+	}
+#endif
+
 	if (!ccache_name) {
 		ccache_name = krb5_cc_default_name(gse_ctx->k5ctx);
 	}
@@ -555,7 +565,7 @@ init_sec_context_done:
 			status = NT_STATUS_NO_MEMORY;
 		}
 
-		gss_maj = gss_release_buffer(&gss_min, &out_data);
+		gss_release_buffer(&gss_min, &out_data);
 	}
 
 done:
@@ -680,7 +690,7 @@ static NTSTATUS gse_get_server_auth_token(TALLOC_CTX *mem_ctx,
 		if (!blob.data) {
 			status = NT_STATUS_NO_MEMORY;
 		}
-		gss_maj = gss_release_buffer(&gss_min, &out_data);
+		gss_release_buffer(&gss_min, &out_data);
 	}
 
 
@@ -732,10 +742,10 @@ static char *gse_errstr(TALLOC_CTX *mem_ctx, OM_uint32 maj, OM_uint32 min)
 
 done:
 	if (msg_min.value) {
-		gss_maj = gss_release_buffer(&gss_min, &msg_min);
+		gss_release_buffer(&gss_min, &msg_min);
 	}
 	if (msg_maj.value) {
-		gss_maj = gss_release_buffer(&gss_min, &msg_maj);
+		gss_release_buffer(&gss_min, &msg_maj);
 	}
 	return errstr;
 }
