@@ -95,7 +95,7 @@ SMB_ACL_T aixacl_sys_acl_get_fd(vfs_handle_struct *handle,
 	/* Get the acl using fstatacl */
    
 	DEBUG(10,("Entering AIX sys_acl_get_fd\n"));
-	DEBUG(10,("fd is %d\n",fsp->fh->fd));
+	DEBUG(10,("fd is %d\n",fsp_get_io_fd(fsp)));
 	file_acl = (struct acl *)SMB_MALLOC(BUFSIZ);
 
 	if(file_acl == NULL) {
@@ -106,7 +106,7 @@ SMB_ACL_T aixacl_sys_acl_get_fd(vfs_handle_struct *handle,
 
 	memset(file_acl,0,BUFSIZ);
 
-	rc = fstatacl(fsp->fh->fd,0,file_acl,BUFSIZ);
+	rc = fstatacl(fsp_get_io_fd(fsp),0,file_acl,BUFSIZ);
 	if( (rc == -1) && (errno == ENOSPC)) {
 		struct acl *new_acl = SMB_MALLOC(file_acl->acl_len + sizeof(struct acl));
 		if( new_acl == NULL) {
@@ -115,7 +115,7 @@ SMB_ACL_T aixacl_sys_acl_get_fd(vfs_handle_struct *handle,
 			return NULL;
 		}
 		file_acl = new_acl;
-		rc = fstatacl(fsp->fh->fd,0,file_acl,file_acl->acl_len + sizeof(struct acl));
+		rc = fstatacl(fsp_get_io_fd(fsp),0,file_acl,file_acl->acl_len + sizeof(struct acl));
 		if( rc == -1) {
 			DEBUG(0,("fstatacl returned %d with errno %d\n",rc,errno));
 			SAFE_FREE(file_acl);
@@ -140,7 +140,7 @@ int aixacl_sys_acl_set_file(vfs_handle_struct *handle,
 {
 	struct acl *file_acl = NULL;
 	unsigned int rc;
-	
+
 	file_acl = aixacl_smb_to_aixacl(type, theacl);
 	if (!file_acl)
 		return -1;
@@ -156,16 +156,26 @@ int aixacl_sys_acl_set_file(vfs_handle_struct *handle,
 
 int aixacl_sys_acl_set_fd(vfs_handle_struct *handle,
 			    files_struct *fsp,
+			    SMB_ACL_TYPE_T type,
 			    SMB_ACL_T theacl)
 {
 	struct acl *file_acl = NULL;
 	unsigned int rc;
 
-	file_acl = aixacl_smb_to_aixacl(SMB_ACL_TYPE_ACCESS, theacl);
+	file_acl = aixacl_smb_to_aixacl(type, theacl);
 	if (!file_acl)
 		return -1;
 
-	rc = fchacl(fsp->fh->fd,file_acl,file_acl->acl_len);
+	if (fsp->fsp_flags.is_pathref) {
+		/*
+		 * This is no longer a handle based call.
+		 */
+		return chacl(fsp->fsp_name->base_name,
+			     file_acl,
+			     file_acl->acl_len);
+	}
+
+	rc = fchacl(fsp_get_io_fd(fsp),file_acl,file_acl->acl_len);
 	DEBUG(10,("errno is %d\n",errno));
 	DEBUG(10,("return code is %d\n",rc));
 	SAFE_FREE(file_acl);
@@ -185,7 +195,6 @@ static struct vfs_fn_pointers vfs_aixacl_fns = {
 	.sys_acl_get_fd_fn = aixacl_sys_acl_get_fd,
 	.sys_acl_blob_get_file_fn = posix_sys_acl_blob_get_file,
 	.sys_acl_blob_get_fd_fn = posix_sys_acl_blob_get_fd,
-	.sys_acl_set_file_fn = aixacl_sys_acl_set_file,
 	.sys_acl_set_fd_fn = aixacl_sys_acl_set_fd,
 	.sys_acl_delete_def_file_fn = aixacl_sys_acl_delete_def_file,
 };
