@@ -171,44 +171,6 @@ static int catia_connect(struct vfs_handle_struct *handle,
 	return SMB_VFS_NEXT_CONNECT(handle, service, user);
 }
 
-static DIR *catia_opendir(vfs_handle_struct *handle,
-			const struct smb_filename *smb_fname,
-			const char *mask,
-			uint32_t attr)
-{
-	char *name_mapped = NULL;
-	NTSTATUS status;
-	DIR *ret;
-	struct smb_filename *mapped_smb_fname = NULL;
-
-	status = catia_string_replace_allocate(handle->conn,
-				smb_fname->base_name,
-				&name_mapped,
-				vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
-		return NULL;
-	}
-
-	mapped_smb_fname = synthetic_smb_fname(talloc_tos(),
-				name_mapped,
-				NULL,
-				&smb_fname->st,
-				smb_fname->flags);
-	if (mapped_smb_fname == NULL) {
-		TALLOC_FREE(mapped_smb_fname);
-		errno = ENOMEM;
-		return NULL;
-	}
-
-	ret = SMB_VFS_NEXT_OPENDIR(handle, mapped_smb_fname, mask, attr);
-
-	TALLOC_FREE(name_mapped);
-	TALLOC_FREE(mapped_smb_fname);
-
-	return ret;
-}
-
 /*
  * TRANSLATE_NAME call which converts the given name to
  * "WINDOWS displayable" name
@@ -476,9 +438,9 @@ static int catia_fetch_fsp_pre_next(TALLOC_CTX *mem_ctx,
 }
 
 #define CATIA_FETCH_FSP_POST_NEXT(_cc, fsp) do { \
-	int saved_errno = errno; \
+	int catia_saved_errno = errno; \
 	catia_fetch_fsp_post_next((_cc), (fsp), __func__); \
-	errno = saved_errno; \
+	errno = catia_saved_errno; \
 } while(0)
 
 static void catia_fetch_fsp_post_next(struct catia_cache **_cc,
@@ -523,21 +485,23 @@ static void catia_fetch_fsp_post_next(struct catia_cache **_cc,
 	return;
 }
 
-static int catia_open(vfs_handle_struct *handle,
-		      struct smb_filename *smb_fname,
-		      files_struct *fsp,
-		      int flags,
-		      mode_t mode)
+static int catia_openat(vfs_handle_struct *handle,
+			const struct files_struct *dirfsp,
+			const struct smb_filename *smb_fname_in,
+			files_struct *fsp,
+			int flags,
+			mode_t mode)
 {
+	struct smb_filename *smb_fname = NULL;
 	struct catia_cache *cc = NULL;
-	char *orig_smb_fname = smb_fname->base_name;
-	char *mapped_smb_fname = NULL;
+	char *mapped_name = NULL;
 	NTSTATUS status;
 	int ret;
+	int saved_errno = 0;
 
 	status = catia_string_replace_allocate(handle->conn,
-					       smb_fname->base_name,
-					       &mapped_smb_fname,
+					       smb_fname_in->base_name,
+					       &mapped_name,
 					       vfs_translate_to_unix);
 	if (!NT_STATUS_IS_OK(status)) {
 		return -1;
@@ -545,17 +509,34 @@ static int catia_open(vfs_handle_struct *handle,
 
 	ret = CATIA_FETCH_FSP_PRE_NEXT(talloc_tos(), handle, fsp, &cc);
 	if (ret != 0) {
-		TALLOC_FREE(mapped_smb_fname);
+		TALLOC_FREE(mapped_name);
 		return ret;
 	}
 
-	smb_fname->base_name = mapped_smb_fname;
-	ret = SMB_VFS_NEXT_OPEN(handle, smb_fname, fsp, flags, mode);
-	smb_fname->base_name = orig_smb_fname;
+	smb_fname = cp_smb_filename(talloc_tos(), smb_fname_in);
+	if (smb_fname == NULL) {
+		TALLOC_FREE(mapped_name);
+		errno = ENOMEM;
+		return -1;
+	}
+	smb_fname->base_name = mapped_name;
 
-	TALLOC_FREE(mapped_smb_fname);
+	ret = SMB_VFS_NEXT_OPENAT(handle,
+				  dirfsp,
+				  smb_fname,
+				  fsp,
+				  flags,
+				  mode);
+
+	if (ret == -1) {
+		saved_errno = errno;
+	}
+	TALLOC_FREE(smb_fname);
+	TALLOC_FREE(mapped_name);
 	CATIA_FETCH_FSP_POST_NEXT(&cc, fsp);
-
+	if (saved_errno != 0) {
+		errno = saved_errno;
+	}
 	return ret;
 }
 
@@ -735,6 +716,7 @@ static int catia_lchown(vfs_handle_struct *handle,
 					name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (catia_smb_fname == NULL) {
 		TALLOC_FREE(name);
@@ -772,6 +754,7 @@ static int catia_chmod(vfs_handle_struct *handle,
 					name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (catia_smb_fname == NULL) {
 		TALLOC_FREE(name);
@@ -809,6 +792,7 @@ static int catia_mkdirat(vfs_handle_struct *handle,
 					name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (catia_smb_fname == NULL) {
 		TALLOC_FREE(name);
@@ -847,6 +831,7 @@ static int catia_chdir(vfs_handle_struct *handle,
 					name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (catia_smb_fname == NULL) {
 		TALLOC_FREE(name);
@@ -913,6 +898,7 @@ catia_realpath(vfs_handle_struct *handle,
 					mapped_name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (catia_smb_fname == NULL) {
 		TALLOC_FREE(mapped_name);
@@ -946,6 +932,7 @@ static int catia_chflags(struct vfs_handle_struct *handle,
 					name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (catia_smb_fname == NULL) {
 		TALLOC_FREE(name);
@@ -991,6 +978,7 @@ catia_streaminfo(struct vfs_handle_struct *handle,
 					mapped_name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (catia_smb_fname == NULL) {
 		TALLOC_FREE(mapped_name);
@@ -1049,18 +1037,20 @@ catia_streaminfo(struct vfs_handle_struct *handle,
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS
-catia_get_nt_acl(struct vfs_handle_struct *handle,
-		 const struct smb_filename *smb_fname,
-		 uint32_t security_info,
-		 TALLOC_CTX *mem_ctx,
-		 struct security_descriptor **ppdesc)
+static NTSTATUS catia_get_nt_acl_at(struct vfs_handle_struct *handle,
+				    struct files_struct *dirfsp,
+				    const struct smb_filename *smb_fname,
+				    uint32_t security_info,
+				    TALLOC_CTX *mem_ctx,
+				    struct security_descriptor **ppdesc)
 {
 	char *mapped_name = NULL;
 	const char *path = smb_fname->base_name;
 	struct smb_filename *mapped_smb_fname = NULL;
 	NTSTATUS status;
 
+	SMB_ASSERT(dirfsp == handle->conn->cwd_fsp);
+
 	status = catia_string_replace_allocate(handle->conn,
 				path, &mapped_name, vfs_translate_to_unix);
 	if (!NT_STATUS_IS_OK(status)) {
@@ -1071,14 +1061,19 @@ catia_get_nt_acl(struct vfs_handle_struct *handle,
 					mapped_name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (mapped_smb_fname == NULL) {
 		TALLOC_FREE(mapped_name);
 		return NT_STATUS_NO_MEMORY;
 	}
 
-	status = SMB_VFS_NEXT_GET_NT_ACL(handle, mapped_smb_fname,
-					 security_info, mem_ctx, ppdesc);
+	status = SMB_VFS_NEXT_GET_NT_ACL_AT(handle,
+					dirfsp,
+					mapped_smb_fname,
+					security_info,
+					mem_ctx,
+					ppdesc);
 	TALLOC_FREE(mapped_name);
 	TALLOC_FREE(mapped_smb_fname);
 
@@ -1110,6 +1105,7 @@ catia_sys_acl_get_file(vfs_handle_struct *handle,
 					mapped_name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (mapped_smb_fname == NULL) {
 		TALLOC_FREE(mapped_name);
@@ -1130,51 +1126,6 @@ catia_sys_acl_get_file(vfs_handle_struct *handle,
 	return ret;
 }
 
-static int
-catia_sys_acl_set_file(vfs_handle_struct *handle,
-			const struct smb_filename *smb_fname,
-			SMB_ACL_TYPE_T type,
-			SMB_ACL_T theacl)
-{
-	struct smb_filename *mapped_smb_fname = NULL;
-	int saved_errno = 0;
-	char *mapped_name = NULL;
-	NTSTATUS status;
-	int ret;
-
-	status = catia_string_replace_allocate(handle->conn,
-				smb_fname->base_name,
-				&mapped_name,
-				vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
-		return -1;
-	}
-
-	mapped_smb_fname = synthetic_smb_fname(talloc_tos(),
-					mapped_name,
-					NULL,
-					&smb_fname->st,
-					smb_fname->flags);
-	if (mapped_smb_fname == NULL) {
-		TALLOC_FREE(mapped_name);
-		errno = ENOMEM;
-		return -1;
-	}
-
-	ret = SMB_VFS_NEXT_SYS_ACL_SET_FILE(handle, mapped_smb_fname,
-			type, theacl);
-	if (ret == -1) {
-		saved_errno = errno;
-	}
-	TALLOC_FREE(mapped_smb_fname);
-	TALLOC_FREE(mapped_name);
-	if (saved_errno != 0) {
-		errno = saved_errno;
-	}
-	return ret;
-}
-
 static int
 catia_sys_acl_delete_def_file(vfs_handle_struct *handle,
 				const struct smb_filename *smb_fname)
@@ -1198,6 +1149,7 @@ catia_sys_acl_delete_def_file(vfs_handle_struct *handle,
 					mapped_name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (mapped_smb_fname == NULL) {
 		TALLOC_FREE(mapped_name);
@@ -1251,6 +1203,7 @@ catia_getxattr(vfs_handle_struct *handle,
 					mapped_name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (mapped_smb_fname == NULL) {
 		TALLOC_FREE(mapped_name);
@@ -1298,6 +1251,7 @@ catia_listxattr(vfs_handle_struct *handle,
 					mapped_name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (mapped_smb_fname == NULL) {
 		TALLOC_FREE(mapped_name);
@@ -1351,6 +1305,7 @@ catia_removexattr(vfs_handle_struct *handle,
 					mapped_name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (mapped_smb_fname == NULL) {
 		TALLOC_FREE(mapped_name);
@@ -1410,6 +1365,7 @@ catia_setxattr(vfs_handle_struct *handle,
 					mapped_name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (mapped_smb_fname == NULL) {
 		TALLOC_FREE(mapped_name);
@@ -1671,6 +1627,7 @@ static int catia_sys_acl_blob_get_fd(vfs_handle_struct *handle,
 
 static int catia_sys_acl_set_fd(vfs_handle_struct *handle,
 				files_struct *fsp,
+				SMB_ACL_TYPE_T type,
 				SMB_ACL_T theacl)
 {
 	struct catia_cache *cc = NULL;
@@ -1681,7 +1638,7 @@ static int catia_sys_acl_set_fd(vfs_handle_struct *handle,
 		return ret;
 	}
 
-	ret = SMB_VFS_NEXT_SYS_ACL_SET_FD(handle, fsp, theacl);
+	ret = SMB_VFS_NEXT_SYS_ACL_SET_FD(handle, fsp, type, theacl);
 
 	CATIA_FETCH_FSP_POST_NEXT(&cc, fsp);
 
@@ -2188,57 +2145,26 @@ static NTSTATUS catia_fsctl(struct vfs_handle_struct *handle,
 	return result;
 }
 
-static NTSTATUS catia_get_compression(vfs_handle_struct *handle,
+static NTSTATUS catia_fget_compression(vfs_handle_struct *handle,
 				      TALLOC_CTX *mem_ctx,
 				      struct files_struct *fsp,
-				      struct smb_filename *smb_fname,
 				      uint16_t *_compression_fmt)
 {
 	NTSTATUS result;
 	struct catia_cache *cc = NULL;
 	int ret;
-	struct smb_filename *mapped_smb_fname = NULL;
-	char *mapped_name = NULL;
 
-	if (fsp != NULL) {
-		ret = CATIA_FETCH_FSP_PRE_NEXT(talloc_tos(), handle, fsp, &cc);
-		if (ret != 0) {
-			return map_nt_error_from_unix(errno);
-		}
-		mapped_smb_fname = fsp->fsp_name;
-	} else {
-		result = catia_string_replace_allocate(handle->conn,
-				smb_fname->base_name,
-				&mapped_name,
-				vfs_translate_to_unix);
-		if (!NT_STATUS_IS_OK(result)) {
-			return result;
-		}
-
-		mapped_smb_fname = synthetic_smb_fname(talloc_tos(),
-						mapped_name,
-						NULL,
-						&smb_fname->st,
-						smb_fname->flags);
-		if (mapped_smb_fname == NULL) {
-			TALLOC_FREE(mapped_name);
-			return NT_STATUS_NO_MEMORY;
-		}
-
-		TALLOC_FREE(mapped_name);
+	ret = CATIA_FETCH_FSP_PRE_NEXT(talloc_tos(), handle, fsp, &cc);
+	if (ret != 0) {
+		return map_nt_error_from_unix(errno);
 	}
 
-	result = SMB_VFS_NEXT_GET_COMPRESSION(handle,
+	result = SMB_VFS_NEXT_FGET_COMPRESSION(handle,
 					mem_ctx,
 					fsp,
-					mapped_smb_fname,
 					_compression_fmt);
 
-	if (fsp != NULL) {
-		CATIA_FETCH_FSP_POST_NEXT(&cc, fsp);
-	} else {
-		TALLOC_FREE(mapped_smb_fname);
-	}
+	CATIA_FETCH_FSP_POST_NEXT(&cc, fsp);
 
 	return result;
 }
@@ -2283,8 +2209,12 @@ static NTSTATUS catia_readdir_attr(struct vfs_handle_struct *handle,
 		return status;
 	}
 
-	smb_fname = synthetic_smb_fname(talloc_tos(), fname, NULL,
-					&smb_fname_in->st, 0);
+	smb_fname = synthetic_smb_fname(talloc_tos(),
+					fname,
+					NULL,
+					&smb_fname_in->st,
+					smb_fname_in->twrp,
+					0);
 
 	status = SMB_VFS_NEXT_READDIR_ATTR(handle, smb_fname, mem_ctx, pattr_data);
 
@@ -2293,44 +2223,6 @@ static NTSTATUS catia_readdir_attr(struct vfs_handle_struct *handle,
 	return status;
 }
 
-static NTSTATUS catia_get_dos_attributes(struct vfs_handle_struct *handle,
-					 struct smb_filename *smb_fname,
-					 uint32_t *dosmode)
-{
-	char *mapped_name = NULL;
-	const char *path = smb_fname->base_name;
-	struct smb_filename *mapped_smb_fname = NULL;
-	NTSTATUS status;
-
-	status = catia_string_replace_allocate(handle->conn,
-				path, &mapped_name, vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
-		return status;
-	}
-	mapped_smb_fname = synthetic_smb_fname(talloc_tos(),
-					mapped_name,
-					NULL,
-					&smb_fname->st,
-					smb_fname->flags);
-	if (mapped_smb_fname == NULL) {
-		TALLOC_FREE(mapped_name);
-		return NT_STATUS_NO_MEMORY;
-	}
-
-	status = SMB_VFS_NEXT_GET_DOS_ATTRIBUTES(handle,
-						 mapped_smb_fname,
-						 dosmode);
-	if (NT_STATUS_IS_OK(status)) {
-		smb_fname->st = mapped_smb_fname->st;
-	}
-
-	TALLOC_FREE(mapped_name);
-	TALLOC_FREE(mapped_smb_fname);
-
-	return status;
-}
-
 static NTSTATUS catia_set_dos_attributes(struct vfs_handle_struct *handle,
 					 const struct smb_filename *smb_fname,
 					 uint32_t dosmode)
@@ -2350,6 +2242,7 @@ static NTSTATUS catia_set_dos_attributes(struct vfs_handle_struct *handle,
 					mapped_name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (mapped_smb_fname == NULL) {
 		TALLOC_FREE(mapped_name);
@@ -2388,6 +2281,7 @@ static NTSTATUS catia_create_dfs_pathat(struct vfs_handle_struct *handle,
 					mapped_name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (mapped_smb_fname == NULL) {
 		TALLOC_FREE(mapped_name);
@@ -2407,7 +2301,7 @@ static NTSTATUS catia_create_dfs_pathat(struct vfs_handle_struct *handle,
 static NTSTATUS catia_read_dfs_pathat(struct vfs_handle_struct *handle,
 			TALLOC_CTX *mem_ctx,
 			struct files_struct *dirfsp,
-			const struct smb_filename *smb_fname,
+			struct smb_filename *smb_fname,
 			struct referral **ppreflist,
 			size_t *preferral_count)
 {
@@ -2428,6 +2322,7 @@ static NTSTATUS catia_read_dfs_pathat(struct vfs_handle_struct *handle,
 					mapped_name,
 					NULL,
 					&smb_fname->st,
+					smb_fname->twrp,
 					smb_fname->flags);
 	if (mapped_smb_fname == NULL) {
 		TALLOC_FREE(mapped_name);
@@ -2440,6 +2335,11 @@ static NTSTATUS catia_read_dfs_pathat(struct vfs_handle_struct *handle,
 					mapped_smb_fname,
 					ppreflist,
 					preferral_count);
+	if (NT_STATUS_IS_OK(status)) {
+		/* Return any stat(2) info. */
+		smb_fname->st = mapped_smb_fname->st;
+	}
+
 	TALLOC_FREE(mapped_name);
 	TALLOC_FREE(mapped_smb_fname);
 	return status;
@@ -2450,11 +2350,10 @@ static struct vfs_fn_pointers vfs_catia_fns = {
 
 	/* Directory operations */
 	.mkdirat_fn = catia_mkdirat,
-	.opendir_fn = catia_opendir,
 	.readdir_attr_fn = catia_readdir_attr,
 
 	/* File operations */
-	.open_fn = catia_open,
+	.openat_fn = catia_openat,
 	.pread_fn = catia_pread,
 	.pread_send_fn = catia_pread_send,
 	.pread_recv_fn = catia_pread_recv,
@@ -2487,19 +2386,18 @@ static struct vfs_fn_pointers vfs_catia_fns = {
 	.strict_lock_check_fn = catia_strict_lock_check,
 	.translate_name_fn = catia_translate_name,
 	.fsctl_fn = catia_fsctl,
-	.get_dos_attributes_fn = catia_get_dos_attributes,
 	.get_dos_attributes_send_fn = vfs_not_implemented_get_dos_attributes_send,
 	.get_dos_attributes_recv_fn = vfs_not_implemented_get_dos_attributes_recv,
 	.set_dos_attributes_fn = catia_set_dos_attributes,
 	.fset_dos_attributes_fn = catia_fset_dos_attributes,
 	.fget_dos_attributes_fn = catia_fget_dos_attributes,
-	.get_compression_fn = catia_get_compression,
+	.fget_compression_fn = catia_fget_compression,
 	.set_compression_fn = catia_set_compression,
 	.create_dfs_pathat_fn = catia_create_dfs_pathat,
 	.read_dfs_pathat_fn = catia_read_dfs_pathat,
 
 	/* NT ACL operations. */
-	.get_nt_acl_fn = catia_get_nt_acl,
+	.get_nt_acl_at_fn = catia_get_nt_acl_at,
 	.fget_nt_acl_fn = catia_fget_nt_acl,
 	.fset_nt_acl_fn = catia_fset_nt_acl,
 
@@ -2507,7 +2405,6 @@ static struct vfs_fn_pointers vfs_catia_fns = {
 	.sys_acl_get_file_fn = catia_sys_acl_get_file,
 	.sys_acl_get_fd_fn = catia_sys_acl_get_fd,
 	.sys_acl_blob_get_fd_fn = catia_sys_acl_blob_get_fd,
-	.sys_acl_set_file_fn = catia_sys_acl_set_file,
 	.sys_acl_set_fd_fn = catia_sys_acl_set_fd,
 	.sys_acl_delete_def_file_fn = catia_sys_acl_delete_def_file,
 
