@@ -72,6 +72,7 @@
 #include "libcli/security/sddl.h"
 #include "passdb/machine_sid.h"
 #include "lib/util/tevent_ntstatus.h"
+#include "lib/util/string_wrappers.h"
 
 static int vfs_full_audit_debug_level = DBGC_VFS;
 
@@ -106,7 +107,6 @@ typedef enum _vfs_op_type {
 
 	/* Directory operations */
 
-	SMB_VFS_OP_OPENDIR,
 	SMB_VFS_OP_FDOPENDIR,
 	SMB_VFS_OP_READDIR,
 	SMB_VFS_OP_SEEKDIR,
@@ -118,6 +118,7 @@ typedef enum _vfs_op_type {
 	/* File operations */
 
 	SMB_VFS_OP_OPEN,
+	SMB_VFS_OP_OPENAT,
 	SMB_VFS_OP_CREATE_FILE,
 	SMB_VFS_OP_CLOSE,
 	SMB_VFS_OP_READ,
@@ -174,14 +175,13 @@ typedef enum _vfs_op_type {
 	SMB_VFS_OP_OFFLOAD_READ_RECV,
 	SMB_VFS_OP_OFFLOAD_WRITE_SEND,
 	SMB_VFS_OP_OFFLOAD_WRITE_RECV,
-	SMB_VFS_OP_GET_COMPRESSION,
+	SMB_VFS_OP_FGET_COMPRESSION,
 	SMB_VFS_OP_SET_COMPRESSION,
 	SMB_VFS_OP_SNAP_CHECK_PATH,
 	SMB_VFS_OP_SNAP_CREATE,
 	SMB_VFS_OP_SNAP_DELETE,
 
 	/* DOS attribute operations. */
-	SMB_VFS_OP_GET_DOS_ATTRIBUTES,
 	SMB_VFS_OP_GET_DOS_ATTRIBUTES_SEND,
 	SMB_VFS_OP_GET_DOS_ATTRIBUTES_RECV,
 	SMB_VFS_OP_FGET_DOS_ATTRIBUTES,
@@ -191,7 +191,7 @@ typedef enum _vfs_op_type {
 	/* NT ACL operations. */
 
 	SMB_VFS_OP_FGET_NT_ACL,
-	SMB_VFS_OP_GET_NT_ACL,
+	SMB_VFS_OP_GET_NT_ACL_AT,
 	SMB_VFS_OP_FSET_NT_ACL,
 	SMB_VFS_OP_AUDIT_FILE,
 
@@ -201,7 +201,6 @@ typedef enum _vfs_op_type {
 	SMB_VFS_OP_SYS_ACL_GET_FD,
 	SMB_VFS_OP_SYS_ACL_BLOB_GET_FILE,
 	SMB_VFS_OP_SYS_ACL_BLOB_GET_FD,
-	SMB_VFS_OP_SYS_ACL_SET_FILE,
 	SMB_VFS_OP_SYS_ACL_SET_FD,
 	SMB_VFS_OP_SYS_ACL_DELETE_DEF_FILE,
 
@@ -253,7 +252,6 @@ static struct {
 	{ SMB_VFS_OP_GET_DFS_REFERRALS,	"get_dfs_referrals" },
 	{ SMB_VFS_OP_CREATE_DFS_PATHAT,	"create_dfs_pathat" },
 	{ SMB_VFS_OP_READ_DFS_PATHAT,	"read_dfs_pathat" },
-	{ SMB_VFS_OP_OPENDIR,	"opendir" },
 	{ SMB_VFS_OP_FDOPENDIR,	"fdopendir" },
 	{ SMB_VFS_OP_READDIR,	"readdir" },
 	{ SMB_VFS_OP_SEEKDIR,   "seekdir" },
@@ -262,6 +260,7 @@ static struct {
 	{ SMB_VFS_OP_MKDIRAT,	"mkdirat" },
 	{ SMB_VFS_OP_CLOSEDIR,	"closedir" },
 	{ SMB_VFS_OP_OPEN,	"open" },
+	{ SMB_VFS_OP_OPENAT,	"openat" },
 	{ SMB_VFS_OP_CREATE_FILE, "create_file" },
 	{ SMB_VFS_OP_CLOSE,	"close" },
 	{ SMB_VFS_OP_READ,	"read" },
@@ -318,26 +317,24 @@ static struct {
 	{ SMB_VFS_OP_OFFLOAD_READ_RECV,	"offload_read_recv" },
 	{ SMB_VFS_OP_OFFLOAD_WRITE_SEND,	"offload_write_send" },
 	{ SMB_VFS_OP_OFFLOAD_WRITE_RECV,	"offload_write_recv" },
-	{ SMB_VFS_OP_GET_COMPRESSION,	"get_compression" },
+	{ SMB_VFS_OP_FGET_COMPRESSION,	"fget_compression" },
 	{ SMB_VFS_OP_SET_COMPRESSION,	"set_compression" },
 	{ SMB_VFS_OP_SNAP_CHECK_PATH, "snap_check_path" },
 	{ SMB_VFS_OP_SNAP_CREATE, "snap_create" },
 	{ SMB_VFS_OP_SNAP_DELETE, "snap_delete" },
-	{ SMB_VFS_OP_GET_DOS_ATTRIBUTES, "get_dos_attributes" },
 	{ SMB_VFS_OP_GET_DOS_ATTRIBUTES_SEND, "get_dos_attributes_send" },
 	{ SMB_VFS_OP_GET_DOS_ATTRIBUTES_RECV, "get_dos_attributes_recv" },
 	{ SMB_VFS_OP_FGET_DOS_ATTRIBUTES, "fget_dos_attributes" },
 	{ SMB_VFS_OP_SET_DOS_ATTRIBUTES, "set_dos_attributes" },
 	{ SMB_VFS_OP_FSET_DOS_ATTRIBUTES, "fset_dos_attributes" },
 	{ SMB_VFS_OP_FGET_NT_ACL,	"fget_nt_acl" },
-	{ SMB_VFS_OP_GET_NT_ACL,	"get_nt_acl" },
+	{ SMB_VFS_OP_GET_NT_ACL_AT,	"get_nt_acl_at" },
 	{ SMB_VFS_OP_FSET_NT_ACL,	"fset_nt_acl" },
 	{ SMB_VFS_OP_AUDIT_FILE,	"audit_file" },
 	{ SMB_VFS_OP_SYS_ACL_GET_FILE,	"sys_acl_get_file" },
 	{ SMB_VFS_OP_SYS_ACL_GET_FD,	"sys_acl_get_fd" },
 	{ SMB_VFS_OP_SYS_ACL_BLOB_GET_FILE,	"sys_acl_blob_get_file" },
 	{ SMB_VFS_OP_SYS_ACL_BLOB_GET_FD,	"sys_acl_blob_get_fd" },
-	{ SMB_VFS_OP_SYS_ACL_SET_FILE,	"sys_acl_set_file" },
 	{ SMB_VFS_OP_SYS_ACL_SET_FD,	"sys_acl_set_fd" },
 	{ SMB_VFS_OP_SYS_ACL_DELETE_DEF_FILE,	"sys_acl_delete_def_file" },
 	{ SMB_VFS_OP_GETXATTR,	"getxattr" },
@@ -921,7 +918,7 @@ static NTSTATUS smb_full_audit_create_dfs_pathat(struct vfs_handle_struct *handl
 static NTSTATUS smb_full_audit_read_dfs_pathat(struct vfs_handle_struct *handle,
 			TALLOC_CTX *mem_ctx,
 			struct files_struct *dirfsp,
-			const struct smb_filename *smb_fname,
+			struct smb_filename *smb_fname,
 			struct referral **ppreflist,
 			size_t *preferral_count)
 {
@@ -989,24 +986,6 @@ static NTSTATUS smb_full_audit_snap_delete(struct vfs_handle_struct *handle,
 	return status;
 }
 
-static DIR *smb_full_audit_opendir(vfs_handle_struct *handle,
-			const struct smb_filename *smb_fname,
-			const char *mask,
-			uint32_t attr)
-{
-	DIR *result;
-
-	result = SMB_VFS_NEXT_OPENDIR(handle, smb_fname, mask, attr);
-
-	do_log(SMB_VFS_OP_OPENDIR,
-	       (result != NULL),
-	       handle,
-	       "%s",
-	       smb_fname_str_do_log(handle->conn, smb_fname));
-
-	return result;
-}
-
 static DIR *smb_full_audit_fdopendir(vfs_handle_struct *handle,
 			  files_struct *fsp, const char *mask, uint32_t attr)
 {
@@ -1021,11 +1000,13 @@ static DIR *smb_full_audit_fdopendir(vfs_handle_struct *handle,
 }
 
 static struct dirent *smb_full_audit_readdir(vfs_handle_struct *handle,
-				    DIR *dirp, SMB_STRUCT_STAT *sbuf)
+					     struct files_struct *dirfsp,
+					     DIR *dirp,
+					     SMB_STRUCT_STAT *sbuf)
 {
 	struct dirent *result;
 
-	result = SMB_VFS_NEXT_READDIR(handle, dirp, sbuf);
+	result = SMB_VFS_NEXT_READDIR(handle, dirfsp, dirp, sbuf);
 
 	/* This operation has no reasonable error condition
 	 * (End of dir is also failure), so always succeed.
@@ -1068,8 +1049,17 @@ static int smb_full_audit_mkdirat(vfs_handle_struct *handle,
 			const struct smb_filename *smb_fname,
 			mode_t mode)
 {
+	struct smb_filename *full_fname = NULL;
 	int result;
 
+	full_fname = full_path_from_dirfsp_atname(talloc_tos(),
+						  dirfsp,
+						  smb_fname);
+	if (full_fname == NULL) {
+		errno = ENOMEM;
+		return -1;
+	}
+
 	result = SMB_VFS_NEXT_MKDIRAT(handle,
 			dirfsp,
 			smb_fname,
@@ -1079,7 +1069,9 @@ static int smb_full_audit_mkdirat(vfs_handle_struct *handle,
 	       (result >= 0),
 	       handle,
 	       "%s",
-	       smb_fname_str_do_log(handle->conn, smb_fname));
+	       smb_fname_str_do_log(handle->conn, full_fname));
+
+	TALLOC_FREE(full_fname);
 
 	return result;
 }
@@ -1096,24 +1088,26 @@ static int smb_full_audit_closedir(vfs_handle_struct *handle,
 	return result;
 }
 
-static int smb_full_audit_open(vfs_handle_struct *handle,
-			       struct smb_filename *smb_fname,
-			       files_struct *fsp, int flags, mode_t mode)
+static int smb_full_audit_openat(vfs_handle_struct *handle,
+				 const struct files_struct *dirfsp,
+				 const struct smb_filename *smb_fname,
+				 struct files_struct *fsp,
+				 int flags,
+				 mode_t mode)
 {
 	int result;
-	
-	result = SMB_VFS_NEXT_OPEN(handle, smb_fname, fsp, flags, mode);
 
-	do_log(SMB_VFS_OP_OPEN, (result >= 0), handle, "%s|%s",
+	result = SMB_VFS_NEXT_OPENAT(handle, dirfsp, smb_fname, fsp, flags, mode);
+
+	do_log(SMB_VFS_OP_OPENAT, (result >= 0), handle, "%s|%s",
 	       ((flags & O_WRONLY) || (flags & O_RDWR))?"w":"r",
-	       smb_fname_str_do_log(handle->conn, smb_fname));
+	       fsp_str_do_log(fsp));
 
 	return result;
 }
 
 static NTSTATUS smb_full_audit_create_file(vfs_handle_struct *handle,
 				      struct smb_request *req,
-				      uint16_t root_dir_fid,
 				      struct smb_filename *smb_fname,
 				      uint32_t access_mask,
 				      uint32_t share_access,
@@ -1160,7 +1154,6 @@ static NTSTATUS smb_full_audit_create_file(vfs_handle_struct *handle,
 	result = SMB_VFS_NEXT_CREATE_FILE(
 		handle,					/* handle */
 		req,					/* req */
-		root_dir_fid,				/* root_dir_fid */
 		smb_fname,				/* fname */
 		access_mask,				/* access_mask */
 		share_access,				/* share_access */
@@ -1807,7 +1800,7 @@ static bool smb_full_audit_getlock(vfs_handle_struct *handle, files_struct *fsp,
 }
 
 static int smb_full_audit_symlinkat(vfs_handle_struct *handle,
-			const char *link_contents,
+			const struct smb_filename *link_contents,
 			struct files_struct *dirfsp,
 			const struct smb_filename *new_smb_fname)
 {
@@ -1822,14 +1815,14 @@ static int smb_full_audit_symlinkat(vfs_handle_struct *handle,
 	       (result >= 0),
 	       handle,
 	       "%s|%s",
-	       link_contents,
+	       link_contents->base_name,
 	       smb_fname_str_do_log(handle->conn, new_smb_fname));
 
 	return result;
 }
 
 static int smb_full_audit_readlinkat(vfs_handle_struct *handle,
-			files_struct *dirfsp,
+			const struct files_struct *dirfsp,
 			const struct smb_filename *smb_fname,
 			char *buf,
 			size_t bufsiz)
@@ -1988,7 +1981,7 @@ static NTSTATUS smb_full_audit_streaminfo(vfs_handle_struct *handle,
 }
 
 static int smb_full_audit_get_real_filename(struct vfs_handle_struct *handle,
-					    const char *path,
+					    const struct smb_filename *path,
 					    const char *name,
 					    TALLOC_CTX *mem_ctx,
 					    char **found_name)
@@ -1999,7 +1992,8 @@ static int smb_full_audit_get_real_filename(struct vfs_handle_struct *handle,
 						found_name);
 
 	do_log(SMB_VFS_OP_GET_REAL_FILENAME, (result == 0), handle,
-	       "%s/%s->%s", path, name, (result == 0) ? *found_name : "");
+	       "%s/%s->%s",
+	       path->base_name, name, (result == 0) ? *found_name : "");
 
 	return result;
 }
@@ -2187,21 +2181,19 @@ static NTSTATUS smb_full_audit_offload_write_recv(struct vfs_handle_struct *hand
 	return result;
 }
 
-static NTSTATUS smb_full_audit_get_compression(vfs_handle_struct *handle,
+static NTSTATUS smb_full_audit_fget_compression(vfs_handle_struct *handle,
 					       TALLOC_CTX *mem_ctx,
 					       struct files_struct *fsp,
-					       struct smb_filename *smb_fname,
 					       uint16_t *_compression_fmt)
 {
 	NTSTATUS result;
 
-	result = SMB_VFS_NEXT_GET_COMPRESSION(handle, mem_ctx, fsp, smb_fname,
+	result = SMB_VFS_NEXT_FGET_COMPRESSION(handle, mem_ctx, fsp,
 					      _compression_fmt);
 
-	do_log(SMB_VFS_OP_GET_COMPRESSION, NT_STATUS_IS_OK(result), handle,
+	do_log(SMB_VFS_OP_FGET_COMPRESSION, NT_STATUS_IS_OK(result), handle,
 	       "%s",
-	       (fsp ? fsp_str_do_log(fsp) :
-		smb_fname_str_do_log(handle->conn, smb_fname)));
+	       fsp_str_do_log(fsp));
 
 	return result;
 }
@@ -2237,26 +2229,6 @@ static NTSTATUS smb_full_audit_readdir_attr(struct vfs_handle_struct *handle,
 	return status;
 }
 
-static NTSTATUS smb_full_audit_get_dos_attributes(
-				struct vfs_handle_struct *handle,
-				struct smb_filename *smb_fname,
-				uint32_t *dosmode)
-{
-	NTSTATUS status;
-
-	status = SMB_VFS_NEXT_GET_DOS_ATTRIBUTES(handle,
-				smb_fname,
-				dosmode);
-
-	do_log(SMB_VFS_OP_GET_DOS_ATTRIBUTES,
-		NT_STATUS_IS_OK(status),
-		handle,
-		"%s",
-		smb_fname_str_do_log(handle->conn, smb_fname));
-
-	return status;
-}
-
 struct smb_full_audit_get_dos_attributes_state {
 	struct vfs_aio_state aio_state;
 	vfs_handle_struct *handle;
@@ -2454,19 +2426,27 @@ static NTSTATUS smb_full_audit_fget_nt_acl(vfs_handle_struct *handle, files_stru
 	return result;
 }
 
-static NTSTATUS smb_full_audit_get_nt_acl(vfs_handle_struct *handle,
-					  const struct smb_filename *smb_fname,
-					  uint32_t security_info,
-					  TALLOC_CTX *mem_ctx,
-					  struct security_descriptor **ppdesc)
+static NTSTATUS smb_full_audit_get_nt_acl_at(vfs_handle_struct *handle,
+				struct files_struct *dirfsp,
+				const struct smb_filename *smb_fname,
+				uint32_t security_info,
+				TALLOC_CTX *mem_ctx,
+				struct security_descriptor **ppdesc)
 {
 	NTSTATUS result;
 
-	result = SMB_VFS_NEXT_GET_NT_ACL(handle, smb_fname, security_info,
-					 mem_ctx, ppdesc);
+	result = SMB_VFS_NEXT_GET_NT_ACL_AT(handle,
+				dirfsp,
+				smb_fname,
+				security_info,
+				mem_ctx,
+				ppdesc);
 
-	do_log(SMB_VFS_OP_GET_NT_ACL, NT_STATUS_IS_OK(result), handle,
-	       "%s", smb_fname_str_do_log(handle->conn, smb_fname));
+	do_log(SMB_VFS_OP_GET_NT_ACL_AT,
+		NT_STATUS_IS_OK(result),
+		handle,
+	       "%s",
+		smb_fname_str_do_log(handle->conn, smb_fname));
 
 	return result;
 }
@@ -2586,31 +2566,14 @@ static int smb_full_audit_sys_acl_blob_get_fd(vfs_handle_struct *handle,
 	return result;
 }
 
-static int smb_full_audit_sys_acl_set_file(vfs_handle_struct *handle,
-				const struct smb_filename *smb_fname,
-				SMB_ACL_TYPE_T acltype,
-				SMB_ACL_T theacl)
+static int smb_full_audit_sys_acl_set_fd(vfs_handle_struct *handle,
+					 struct files_struct *fsp,
+					 SMB_ACL_TYPE_T type,
+					 SMB_ACL_T theacl)
 {
 	int result;
 
-	result = SMB_VFS_NEXT_SYS_ACL_SET_FILE(handle, smb_fname, acltype,
-					       theacl);
-
-	do_log(SMB_VFS_OP_SYS_ACL_SET_FILE,
-	       (result >= 0),
-	       handle,
-	       "%s",
-	       smb_fname_str_do_log(handle->conn, smb_fname));
-
-	return result;
-}
-
-static int smb_full_audit_sys_acl_set_fd(vfs_handle_struct *handle, files_struct *fsp,
-				SMB_ACL_T theacl)
-{
-	int result;
-
-	result = SMB_VFS_NEXT_SYS_ACL_SET_FD(handle, fsp, theacl);
+	result = SMB_VFS_NEXT_SYS_ACL_SET_FD(handle, fsp, type, theacl);
 
 	do_log(SMB_VFS_OP_SYS_ACL_SET_FD, (result >= 0), handle,
 	       "%s", fsp_str_do_log(fsp));
@@ -2990,7 +2953,6 @@ static struct vfs_fn_pointers vfs_full_audit_fns = {
 	.get_dfs_referrals_fn = smb_full_audit_get_dfs_referrals,
 	.create_dfs_pathat_fn = smb_full_audit_create_dfs_pathat,
 	.read_dfs_pathat_fn = smb_full_audit_read_dfs_pathat,
-	.opendir_fn = smb_full_audit_opendir,
 	.fdopendir_fn = smb_full_audit_fdopendir,
 	.readdir_fn = smb_full_audit_readdir,
 	.seekdir_fn = smb_full_audit_seekdir,
@@ -2998,7 +2960,7 @@ static struct vfs_fn_pointers vfs_full_audit_fns = {
 	.rewind_dir_fn = smb_full_audit_rewinddir,
 	.mkdirat_fn = smb_full_audit_mkdirat,
 	.closedir_fn = smb_full_audit_closedir,
-	.open_fn = smb_full_audit_open,
+	.openat_fn = smb_full_audit_openat,
 	.create_file_fn = smb_full_audit_create_file,
 	.close_fn = smb_full_audit_close,
 	.pread_fn = smb_full_audit_pread,
@@ -3044,7 +3006,7 @@ static struct vfs_fn_pointers vfs_full_audit_fns = {
 	.offload_read_recv_fn = smb_full_audit_offload_read_recv,
 	.offload_write_send_fn = smb_full_audit_offload_write_send,
 	.offload_write_recv_fn = smb_full_audit_offload_write_recv,
-	.get_compression_fn = smb_full_audit_get_compression,
+	.fget_compression_fn = smb_full_audit_fget_compression,
 	.set_compression_fn = smb_full_audit_set_compression,
 	.snap_check_path_fn =  smb_full_audit_snap_check_path,
 	.snap_create_fn = smb_full_audit_snap_create,
@@ -3057,21 +3019,19 @@ static struct vfs_fn_pointers vfs_full_audit_fns = {
 	.strict_lock_check_fn = smb_full_audit_strict_lock_check,
 	.translate_name_fn = smb_full_audit_translate_name,
 	.fsctl_fn = smb_full_audit_fsctl,
-	.get_dos_attributes_fn = smb_full_audit_get_dos_attributes,
 	.get_dos_attributes_send_fn = smb_full_audit_get_dos_attributes_send,
 	.get_dos_attributes_recv_fn = smb_full_audit_get_dos_attributes_recv,
 	.fget_dos_attributes_fn = smb_full_audit_fget_dos_attributes,
 	.set_dos_attributes_fn = smb_full_audit_set_dos_attributes,
 	.fset_dos_attributes_fn = smb_full_audit_fset_dos_attributes,
 	.fget_nt_acl_fn = smb_full_audit_fget_nt_acl,
-	.get_nt_acl_fn = smb_full_audit_get_nt_acl,
+	.get_nt_acl_at_fn = smb_full_audit_get_nt_acl_at,
 	.fset_nt_acl_fn = smb_full_audit_fset_nt_acl,
 	.audit_file_fn = smb_full_audit_audit_file,
 	.sys_acl_get_file_fn = smb_full_audit_sys_acl_get_file,
 	.sys_acl_get_fd_fn = smb_full_audit_sys_acl_get_fd,
 	.sys_acl_blob_get_file_fn = smb_full_audit_sys_acl_blob_get_file,
 	.sys_acl_blob_get_fd_fn = smb_full_audit_sys_acl_blob_get_fd,
-	.sys_acl_set_file_fn = smb_full_audit_sys_acl_set_file,
 	.sys_acl_set_fd_fn = smb_full_audit_sys_acl_set_fd,
 	.sys_acl_delete_def_file_fn = smb_full_audit_sys_acl_delete_def_file,
 	.getxattr_fn = smb_full_audit_getxattr,
